class Socioeconomico < ActiveRecord::Base
  belongs_to :ocupacao
  belongs_to :escolaridade
  belongs_to :religiao
  belongs_to :tipohabitacao
  belongs_to :situacaofamiliar
  belongs_to :estadomarital
  belongs_to :pessoa

  validates_numericality_of :tempomoradia, :only_integer => true, :allow_nil => true, :greater_than => 0, :less_than_or_equal_to=> 999

  validates_numericality_of :tempoendereco, :only_integer => true, :allow_nil => true, :greater_than => 0, :less_than_or_equal_to=> 999

  validates_date :dataentradapais, :allow_nil => true, :message => " inválida, verifique o formato DD/MM/AAAA e os valores digitados."

  validates_format_of :outrasatividades, :allow_blank => true, :with => /^([a-z]|[A-Z]|[ ]|[á-ú]|[â-û]|[ã-ũ]|[Á-Ú]|[Â-Û]|[Ã-Ũ]|[à-ù]|[À-Ù])+$/, :message => " pode apenas conter letras."
  validates_length_of :outrasatividades, :allow_blank => true, :maximum=>35, :too_long => " pode ter no máximo %d caracteres."

  validates_numericality_of :rendaindividual, :only_integer => true, :allow_blank => true, :greater_than => 0, :less_than_or_equal_to=> 99999999

  validates_numericality_of :rendadomicilio, :only_integer => true, :allow_blank => true, :greater_than => 0, :less_than_or_equal_to=> 99999999

  validates_format_of :nomeescola, :allow_blank => true, :with => /^([a-z]|[A-Z]|[ ]|[á-ú]|[â-û]|[ã-ũ]|[Á-Ú]|[Â-Û]|[Ã-Ũ]|[à-ù]|[À-Ù]|[0-9])+$/, :message => " pode apenas conter letras."
  validates_length_of :nomeescola, :allow_blank => true, :maximum=>35, :too_long => " pode ter no máximo %d caracteres."

  validates_numericality_of :anosescola, :only_integer => true, :allow_nil => true, :greater_than => 0, :less_than_or_equal_to=> 999

  validates_numericality_of :quantopaga, :only_integer => true, :allow_blank => true, :greater_than => 0, :less_than_or_equal_to=> 99999999

  validates_numericality_of :qtdepessoas, :only_integer => true, :allow_nil => true, :greater_than => 0, :less_than_or_equal_to=> 99

  validates_numericality_of :qtdecomodos, :only_integer => true, :allow_nil => true, :greater_than => 0, :less_than_or_equal_to=> 99


  def to_label
   self.created_at.to_s
  end


  def ocupacao_codigo
    ocupacao.codigo if ocupacao
  end

  def ocupacao_codigo=(nome)
    self.ocupacao = Ocupacao.find_by_codigo(nome) unless nome.blank?
  end

  def to_s
    "socioeconomico"
  end

end
